# Generated by Django 3.0.1 on 2019-12-30 13:05

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0011_auto_20191230_1305'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='travelprotection',
            name='protect_components',
        ),
        migrations.RemoveField(
            model_name='travelprotection',
            name='protect_contents',
        ),
        migrations.DeleteModel(
            name='TravelProtection',
        ),
        migrations.DeleteModel(
            name='LinkContent',
        ),
        migrations.RemoveField(
            model_name='assignedcontent',
            name='object_id',
        ),
        migrations.CreateModel(
            name='LinkContent',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('spider_base.datacontent',),
        ),
        migrations.CreateModel(
            name='TravelProtection',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('spider_base.datacontent',),
        ),
        migrations.AlterModelOptions(
            name='travelprotection',
            options={'permissions': [('use_dangerous_travelprotections', 'Can use dangerous TravelProtections')]},
        ),
    ]
