# Generated by Django 2.2.5 on 2019-09-29 21:17

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('spider_verifier', '0002_20190530_squashed'),
    ]

    operations = [
        migrations.AlterField(
            model_name='verifysourceobject',
            name='token',
            field=models.CharField(blank=True, max_length=255, null=True, unique=True),
        ),
    ]
