# Generated by Django 3.0.1 on 2019-12-30 13:51

from django.db import migrations


class Migration(migrations.Migration):

    dependencies = [
        ('spider_base', '0012_auto_20191230_1305'),
        ('spider_webcfg', '0004_auto_20191230_1351'),
    ]

    operations = [
        migrations.DeleteModel(
            name='WebConfig',
        ),
        migrations.CreateModel(
            name='WebConfig',
            fields=[
            ],
            options={
                'proxy': True,
                'indexes': [],
                'constraints': [],
            },
            bases=('spider_base.datacontent',),
        ),
    ]
