{% load i18n %}
<div{% include 'django/forms/widgets/attrs.html' %}>
{% for entry in summary %}
<strong>{{ entry.label }}</strong>{% if entry.value %}: {{ entry.value }}{% endif %}
{% endfor %}
<div>
  <strong><a href="{% url 'auth:password_change' %}">{% trans 'Change Password' %}</a></strong>
</div>
</div>
